# KeepPressingString.ps1
# Keep pressing specified strings, asking every so often whether it should stop
# Created   2006-09-25 by Nathan Tuggy
# Converted 2007-03-24 by Nathan Tuggy from VBScript
# Modified  2007-10-16 by Nathan Tuggy
# Version 1.1.0.02
# Runs with Windows PowerShell 1.0

# Copyright 2007 Nathan Tuggy <mailto:nathan@tuggycomputer.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

param ([String[]]$Commands = @(), [Double]$InitialPause = 5, [Double]$CommandPause = .1, [Double]$GroupPause = .2`
 , [Int32]$GroupsPerBatch = 1, [Int32]$BatchPause = 1, [Int32]$BatchCount = 1`
 , [Switch]$Help)

if (-not [System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")) {
    Write-Error "Unable to load required assembly System.Windows.Forms"
    break
}

Set-Variable lngcExCodeNormal 0 -option constant
Set-Variable lngcExCodeBadCmdLine 1 -option constant
& {
    Set-Variable strcChoiceTypeName "System.Management.Automation.Host.ChoiceDescription" -option constant
    $chcYes = New-Object $strcChoiceTypeName "&Yes", "Send another batch of keystrokes and ask again"
    $chcAll = New-Object $strcChoiceTypeName "Yes to &All", "Continue sending keystrokes and don't ask again"
    $chcNo = New-Object $strcChoiceTypeName "&No", "Stop sending keystrokes and exit the script"
    $chcPause = New-Object $strcChoiceTypeName "&Pause", "Pause for $BatchPause seconds before asking again"
    #$chcTen = New-Object $strcChoiceTypeName("&Ten", "Send ten batches of keystrokes and ask again")
    $script:achc = [System.Management.Automation.Host.ChoiceDescription[]]($chcYes, $chcAll, $chcNo, $chcPause)
}

function NewParameterDescription {
    param ([String]$Parameter = "", [String]$Description = "")
    
    if ($Parameter.Length -eq 0) {
        throw "You must specify the Parameter argument."
    }
    if ($Description.Length -eq 0) {
        throw "You must specify the Description argument."
    }
    return New-Object System.Object | Add-Member NoteProperty Parameter $Parameter -PassThru | Add-Member NoteProperty Description $Description -PassThru
}

function ShowUsage {
    [Object[]]$aobjDescriptions = @()
    
    # Print main usage string
    #[String]$strUsageHeader = "Usage: KeepPressingString.ps1 -Commands [String[]] [-InitialPause [Double]] [-CommandPause [Double]]"
    #$strUsageHeader += " [-GroupPause [Double]] [-GroupsPerBatch [Int32]] [-BatchPause [Int32]] [-BatchCount [Int32]] [-Help]"
    #Write-Output $strUsageHeader
    "NAME`n    KeepPressingString.ps1"
    "`nSYNOPSIS`n    Keep pressing specified command strings, asking every so often whether it should stop.`n"
    "SYNTAX`n    KeepPressingString.ps1 -Commands <String[]> [-InitialPause <Double>] [-CommandPause <Double>] [-GroupPause <Double>] [-GroupsPerBatch <Int32>] [-BatchPause <Int32>] [-BatchCount <Int32>] [-Help]`n"
    "DETAILED DESCRIPTION`n    Sends keystrokes to the active window, using the Win32 SendKeys function (See MSDN or Windows Script Host documentation). KeepPressingString pauses between commands, groups of commands, and batches of groups, as well as before starting, to allow switching windows."
    "    CAUTION: KeepPressingString can be quite dangerous.`n"
    
    "PARAMETERS"
    
    # Print parameter descriptions -- non-standard, but oh well
    $aobjDescriptions += NewParameterDescription "-Commands <String[]>" `
     "Array of strings to keep sending. (Note: To send the Enter key, type {enter}; Ctrl is ^, Alt is %, and Shift is +. For further information, see the Windows Script Host Documentation for SendKeys)"
    $aobjDescriptions += NewParameterDescription "-InitialPause <Double>" "Time to wait before beginning to send strings"
    $aobjDescriptions += NewParameterDescription "-CommandPause <Double>" "Time between each string send, in seconds"
    $aobjDescriptions += NewParameterDescription "-GroupPause <Double>" "Time to pause between each string group, in seconds"
    $aobjDescriptions += NewParameterDescription "-GroupsPerBatch <Int32>" "Number of string groups to send between BatchPauses"
    $aobjDescriptions += NewParameterDescription "-BatchPause <Int32>" "Time to wait between each batch of string groups"
    $aobjDescriptions += NewParameterDescription "-BatchCount <Int32>" `
     "Number of batches to send. If BatchCount is zero, prompts after every batch and continues indefinitely. If BatchCount is less than zero, never prompts and continues indefinitely"
    $aobjDescriptions += NewParameterDescription "-Help" "Displays this message. (Note: -? will NOT work)"
    
    Write-Output $aobjDescriptions | Format-Table -AutoSize -HideTableHeaders -Wrap
    
    ""
}

function Pause ([Double]$Pause = $(throw (New-Object InvalidArgumentException "Missing required parameter", "Pause"))) {
    Write-Verbose "Pausing for $Pause seconds."
    Start-Sleep -Milliseconds ($Pause * 1000)
}

# Check parameter validity
if ($Help.IsPresent) {
    ShowUsage
    break
} elseif ([Int32]$Commands.Length -eq 0) {
    Write-Error "You must specify one or more command strings to send."
    break
} elseif ($InitialPause -lt 0) {
    Write-Error "InitialPause must be at least 0."
    break
} elseif ($CommandPause -lt 0) {
    Write-Error "CommandPause must be at least 0."
    break
} elseif ($GroupPause -lt 0) {
    Write-Error "GroupPause must be at least 0."
    break
} elseif ($GroupsPerBatch -lt 1) {
    Write-Error "GroupsPerBatch must be at least 1."
    break
} elseif ($BatchPause -lt 0) {
    Write-Error "BatchPause must be at least 0."
    break
}

Pause $InitialPause
Write-Debug "`$BatchCount: $BatchCount"
for ([Int32]$bintBatches = 1; $BatchCount -le 0 -or $bintBatches -le $BatchCount; $bintBatches++) {
    for ([Int32]$bi = 1; $bi -le $GroupsPerBatch; $bi++) {
        foreach ($bstrCommand In $Commands) {
            Write-Debug "Sending $bstrCommand"
            [System.Windows.Forms.SendKeys]::SendWait($bstrCommand)
            Pause $CommandPause
        }
    }
    # If $BatchCount is not equal to zero, it will never ask you
    if ($BatchCount -eq 0) {
        # Prompt every time
        [Boolean]$bblnExit = $False
        while (-not $bblnExit) {
            switch ($host.UI.PromptForChoice("Continue", "Do you want to continue?", $achc, 0)) {
                0 { # Yes
                    Pause $InitialPause   # Wait for user to switch back to whatever application they're sending keys to
                    $bblnExit = $True
                }
                1 { # Yes to All
                    Pause $InitialPause   # Wait for user to switch back to whatever application they're sending keys to
                    $bblnExit = $True
                    $BatchCount = -1
                }
                2 { # No
                    exit
                }
                3 { # Pause
                    Pause $BatchPause
                }
            }
        }
    } else { Pause $BatchPause }
    Write-Debug "`$bintBatches: $bintBatches"
}
